/*
 *  Copyright (C) 2021 Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
    MCU_SPI0_D0 is your COPI(Controller Out Peripheral In). This should be connected to ADS8166 SDI.
    MCU_SPI0_D1 is your CIPO(Controller In Peripheral Out). This should be connected to ADS8166 SDO.
    MCU_SPI0_CS1 is your chip select and should be connected to ADS8166 CS line.
    MCU_SPI0_CLK should be connected to ADS8166 CLK line.
*/

#include <kernel/dpl/DebugP.h>
#include <kernel/dpl/ClockP.h>
#include "ti_drivers_config.h"
#include "ti_drivers_open_close.h"
#include "ti_board_open_close.h"
#include <drivers/gpio.h>
#include <kernel/dpl/AddrTranslateP.h>

#define APP_MCSPI_MSGSIZE                   (1U)

uint32_t gMcspiTxBuffer[APP_MCSPI_MSGSIZE];
uint32_t gMcspiRxBuffer[APP_MCSPI_MSGSIZE];

//static void i2c_read_error_handler(int32_t status);

void empty_main_i2c(void *arg0)
{
    int32_t         status = SystemP_SUCCESS;
    DebugP_log("\nstatus: %c", status);
    I2C_Handle      i2cHandle;
    uint8_t buffertowrite[3];
    uint8_t buffertowrite2[2];
    uint8_t buffertowrite1[1];
    uint8_t readBuffer[1];
    //uint32_t    loopcnt = 10, delayMsec = 3000U;

    /* STEP 1:
     *
     * Opening drivers, I2C being one of them.
     */

    /* STEP 2:
     *
     * Initializes I2C Handle.
     */
    i2cHandle = gI2cHandle[CONFIG_I2C0];
    
    // Detect all I2C devices
    //detect_all_i2c_devices(i2cHandle);
    
     /* STEP 3:
     *
     * Suppose our usecase is to set PIN-01 and PIN-05 to Output Mode, then we need to
     * write to Configuration Registers more specifically Configuration Port 0.
     * Configuration Port 0 is at 0x0C and the default value of this port is b"11111111"
     * We need to set the 0th bit, 2nd bit and 5th to 0. So the value to write would be b"11011010" which is 0xDA.
     */

// write config - configure Port 0 as output and Port 1 as input
        buffertowrite[0] = 0x8C;
        buffertowrite[1] = 0x00;
        buffertowrite[2] = 0xFF;
        I2C_Transaction i2cTransaction;
        I2C_Transaction_init(&i2cTransaction);
        i2cTransaction.slaveAddress = 0x22;
        i2cTransaction.writeBuf = buffertowrite;
        i2cTransaction.writeCount = 3;
        status = I2C_transfer(i2cHandle, &i2cTransaction);
        DebugP_assert(SystemP_SUCCESS == status);
        DebugP_log("\nTask2 - Set port0 as output for power switch and port1 as input from octocoupler");


//read the port - if any of the 8 pins of port 1 are low then turn corresponding port0 pins high
	while(1){
    	buffertowrite1[0] = 0x01;
//      I2C_Transaction_init(&i2cTransaction);
//      i2cTransaction.slaveAddress = 0x22; // I2C slave address
        i2cTransaction.writeBuf = buffertowrite1;
        i2cTransaction.writeCount = 1; // Writing only the register address
        i2cTransaction.readBuf = readBuffer; // Buffer to store the read data
        i2cTransaction.readCount = 1; // Reading 1 byte
    // Perform the I2C read operation continously
    status = I2C_transfer(i2cHandle, &i2cTransaction);
    DebugP_assert(SystemP_SUCCESS == status);
        DebugP_log("\nTask2loop - Read value from register 0x01: 0x%02X\n", readBuffer[0]);
// write the negated value of input status register of port1 to output status register of port 0
        buffertowrite2[0] = 0x04;
        buffertowrite2[1] = ~readBuffer[0];
        i2cTransaction.writeBuf = buffertowrite2;
        i2cTransaction.writeCount = 2;
        status = I2C_transfer(i2cHandle, &i2cTransaction);
        DebugP_assert(SystemP_SUCCESS == status);
        DebugP_log("\nTask2loop - Data written in 0x04 is 0x%02X\n", buffertowrite2[1]);
    }
}
/*
static void i2c_read_error_handler(int32_t status)
{
    switch(status)
    {
        case I2C_STS_ERR:
            DebugP_logError("\nGeneric error occurred");
            break;
        case I2C_STS_ERR_TIMEOUT:
            DebugP_logError("\nTimeout error occurred");
            break;
        case I2C_STS_ERR_NO_ACK:
            DebugP_logError("\nNo acknowledgement received\n");
            break;
        case I2C_STS_ERR_ARBITRATION_LOST:
            DebugP_logError("\nSample Arbitration lost\n");
            break;
        case I2C_STS_ERR_ACCESS_ERROR:
            DebugP_logError("\nBus Access error occurred\n");
            break;
    }

    return;
}
*/

void gpio_led_blink_main(void *args)
{
    uint32_t    loopcnt = 5, delaySec = 1;
    uint32_t    gpioBaseAddr, pinNum;

    DebugP_log("Task3 - GPIO LED Blink Test Started ...\r\n");
    DebugP_log("LED will Blink for %d seconds ...\r\n", (loopcnt * delaySec * 2));

    /* Get address after translation translate */
    gpioBaseAddr = (uint32_t) AddrTranslateP_getLocalAddr(GPIO_LED_BASE_ADDR);
    pinNum       = GPIO_LED_PIN;
    DebugP_log("Task3 - pinNum:%d\r\n",pinNum);
    GPIO_setDirMode(gpioBaseAddr, pinNum, GPIO_LED_DIR);
    while(loopcnt > 0)
    {
        GPIO_pinWriteHigh(gpioBaseAddr, pinNum);
        DebugP_log("Task3loop - LED High!!\r\n");
        ClockP_sleep(delaySec);
        GPIO_pinWriteLow(gpioBaseAddr, pinNum);
        DebugP_log("Task3loop - LED low!!\r\n");
        ClockP_sleep(delaySec);

        loopcnt--;
    }

    DebugP_log("Task3Finish - GPIO LED Blink Test Passed!!\r\n");
    DebugP_log("Task3Finish - All tests have passed!!\r\n");
}

void *empty_main_spi(void *args)
{
    uint8_t            i;
    int32_t             transferOK;
    MCSPI_Transaction   spiTransaction;

    DebugP_log("Task1 - ADS8166 Interfacing with MCSPI ...\r\n");

    /* Memfill buffers */
    for(i = 0U; i < APP_MCSPI_MSGSIZE; i++)
    {
        gMcspiTxBuffer[i] = 0U;
        gMcspiRxBuffer[i] = 0U;
    }

    /* 
    dataToWrite buffer has the following commands:
    -> 0x0800AA - reg access 
    -> 0x08041C - disable internal ref 
    -> 0x081010 - Data output format set
    -> 0x081802 - offset callib 
    -> 0x081C00 - Device config manual default 
    -> 0x081D04 - selects analog input for manual mode
    */
    uint32_t dataToWrite[7] = {0x0800AA, 0x08041C, 0x081020, 0x081802, 0x081C00, 0x081D03};
    for(int i = 0; i < 7; i++){

        // intialize the tx buffer
        gMcspiTxBuffer[0] = dataToWrite[i];

        /* define the transfer parameters */
        MCSPI_Transaction_init(&spiTransaction);
        spiTransaction.channel  = gConfigMcspi0ChCfg[0].chNum;
        spiTransaction.dataSize = 32;
        spiTransaction.csDisable = TRUE;
        spiTransaction.count    = 1;
        spiTransaction.txBuf    = (void *)gMcspiTxBuffer;
        spiTransaction.rxBuf    = (void *)gMcspiRxBuffer;
        spiTransaction.args     = NULL;

        // initiate the MCSPI transfer
        transferOK = MCSPI_transfer(gMcspiHandle[CONFIG_MCSPI0], &spiTransaction);


        if((SystemP_SUCCESS != transferOK) ||
        (MCSPI_TRANSFER_COMPLETED != spiTransaction.status))
        {
            DebugP_log("Task1 - Data 0x%08X is not written!! \r\n", dataToWrite[i]);
        }
        else
        {
            DebugP_log("Task1 - Data 0x%08X written successfully!! \r\n", dataToWrite[i]);
        }
        
        DebugP_log("gMcspiRxBuffer = 0x%08X \r\n", gMcspiRxBuffer[0]);
    }


//send nop commands continously
    while(1){

        // intialize the tx buffer
        gMcspiTxBuffer[0] = 0x0;

        /* define the transfer parameters */
        MCSPI_Transaction_init(&spiTransaction);
        spiTransaction.channel  = gConfigMcspi0ChCfg[0].chNum;
        spiTransaction.dataSize = 32;
        spiTransaction.csDisable = TRUE;
        spiTransaction.count    = 1;
        spiTransaction.txBuf    = (void *)gMcspiTxBuffer;
        spiTransaction.rxBuf    = (void *)gMcspiRxBuffer;
        spiTransaction.args     = NULL;

        // initiate the MCSPI transfer
        transferOK = MCSPI_transfer(gMcspiHandle[CONFIG_MCSPI0], &spiTransaction);


        if((SystemP_SUCCESS != transferOK) ||
        (MCSPI_TRANSFER_COMPLETED != spiTransaction.status))
        {
            DebugP_log("Task1loop - Data 0x%08X is not written!! \r\n", gMcspiTxBuffer[0]);
        }
        else
        {
            DebugP_log("Task1loop - Data 0x%08X written successfully!! \r\n", gMcspiTxBuffer[0]);
        }
        
        DebugP_log("Task1loop - gMcspiRxBuffer = 0x%08X \r\n", gMcspiRxBuffer[0]);
    }


    return NULL;
}


